<?php
namespace nulib\cache;

use IteratorAggregate;
use nulib\cl;
use nulib\db\CapacitorChannel;
use nulib\db\CapacitorStorage;
use nulib\ext\utils;
use nulib\php\func;
use Traversable;

class CacheChannel extends CapacitorChannel implements IteratorAggregate {
  const NAME = "cache";
  const TABLE_NAME = "cache";

  const COLUMN_DEFINITIONS = [
    "group_id" => "varchar(32) not null", // groupe de curseur
    "id" => "varchar(128) not null", // nom du curseur
    "key_index" => "integer not null",
    "key" => "varchar(128) not null",
    "search" => "varchar(255)",

    "primary key (group_id, id, key_index)",
  ];

  static function with(CapacitorStorage $storage, ?iterable $rows=null, $cursorId=null): self {
    $channel = (new static($cursorId))->initStorage($storage);
    if ($rows !== null) $channel->build($rows);
    return $channel;
  }

  static function verifix_id(&$cursorId): void {
    $cursorId ??= utils::uuidgen();
    if (is_array($cursorId)) {
      $keys = array_keys($cursorId);
      if (array_key_exists("group_id", $cursorId)) $groupIdKey = "group_id";
      else $groupIdKey = $keys[1] ?? null;
      $groupId = strval($cursorId[$groupIdKey] ?? "");
      if (array_key_exists("id", $cursorId)) $idKey = "id";
      else $idKey = $keys[0] ?? null;
      $id = strval($cursorId[$idKey] ?? "");
    } else {
      $groupId = "";
      $id = strval($cursorId);
    }
    # si le groupe ou le nom sont trop grand, en faire un hash
    if (strlen($groupId) > 32) {
      $groupId = md5($groupId);
    }
    if (strlen($id) > 128) {
      $id = substr($id, 0, 128 - 32).md5($id);
    }
    $cursorId = ["group_id" => $groupId, "id" => $id];
  }

  /**
   * @param array|string $cursorId
   */
  function __construct($cursorId) {
    parent::__construct();
    self::verifix_id($cursorId);
    $this->cursorId = $cursorId;
  }

  protected array $cursorId;

  function getCursorId(): array {
    return $this->cursorId;
  }

  function getBaseFilter(): ?array {
    return $this->cursorId;
  }

  protected int $index = 0;

  protected function getSearch($item): ?string {
    $search = cl::filter_n(cl::with($item));
    $search = implode(" ", $search);
    return substr($search, 0, 255);
  }

  function getItemValues($item, $key=null): ?array {
    $index = $this->index++;
    $key = $key ?? $index;
    $key = substr(strval($key), 0, 128);
    return cl::merge($this->cursorId, [
      "key_index" => $index,
      "key" => $key,
      "search" => $this->getSearch($item),
    ]);
  }

  function reset(bool $recreate=false): void {
    $this->index = 0;
    parent::reset($recreate);
  }

  function chargeAll(?iterable $items, $func=null, ?array $args=null): int {
    $this->index = 0;
    if ($items === null) return 0;
    $count = 0;
    if ($func !== null) $func = func::with($func, $args)->bind($this);
    foreach ($items as $key => $item) {
      $count += $this->charge($item, $func, [$key]);
    }
    return $count;
  }

  function build(?iterable $items): self {
    $this->reset(true);
    $this->chargeAll($items);
    return $this;
  }

  function getIterator(): Traversable {
    $rows = $this->dbAll([
      "cols" => ["key", "item__"],
      "where" => $this->getBaseFilter(),
    ]);
    foreach ($rows as $row) {
      $key = $row["key"];
      $item = $this->unserialize($row["item__"]);
      yield $key => $item;
    }
  }
}
